package com.fjhx.utils;

import cn.hutool.core.util.StrUtil;
import com.aliyun.dingtalkoauth2_1_0.Client;
import com.aliyun.dingtalkoauth2_1_0.models.GetCorpAccessTokenRequest;
import com.aliyun.dingtalkoauth2_1_0.models.GetCorpAccessTokenResponseBody;
import com.aliyun.tea.TeaException;
import com.aliyun.teaopenapi.models.Config;
import com.dingtalk.api.DefaultDingTalkClient;
import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.OapiV2UserGetuserinfoRequest;
import com.dingtalk.api.response.OapiV2UserGetuserinfoResponse;
import com.fjhx.constant.Constant;
import com.ruoyi.common.exception.ServiceException;
import com.taobao.api.ApiException;
import lombok.extern.slf4j.Slf4j;


/**
 * 钉钉第三方企业应用 - h5微应用
 *
 * <p>
 * <a href="https://open.dingtalk.com/document/isvapp/basic-concepts"> 基础概念 </a>
 * <a href="https://open.dingtalk.com/document/isvapp/api-overview"> 钉钉api总览 </a>
 * <a href="https://open.dingtalk.com/document/isvapp/third-party-enterprise-application-logon-free"> 免登流程 </a>
 * </p>
 */

@Slf4j
public class DingUtil {

    /**
     * 获取第三方应用授权企业的accessToken
     * https://open.dingtalk.com/document/isvapp/obtain-the-access_token-of-the-authorized-enterprise
     */
    public static GetCorpAccessTokenResponseBody getCorpAccessToken(String corpId) {

        GetCorpAccessTokenRequest getCorpAccessTokenRequest = new GetCorpAccessTokenRequest()
                .setSuiteKey(Constant.SUITE_KEY)
                .setSuiteSecret(Constant.SUITE_SECRET)
                .setAuthCorpId(corpId)
                .setSuiteTicket("Kn3hJvJR34e8rUEob0W7nqxK2YHqobAyPsuIQXYR35SsJMp6rE5XQBz9nwXTrJRZaVmXc4z9KjYkzXdkUdS8O3");

        try {
            Client client = getClient2();
            GetCorpAccessTokenResponseBody corpAccessToken = client.getCorpAccessToken(getCorpAccessTokenRequest).getBody();
            return corpAccessToken;
        } catch (Exception e) {

            TeaException teaException;

            if (e instanceof TeaException) {
                teaException = (TeaException) e;
            } else {
                teaException = new TeaException(e.getMessage(), e);
            }

            String errCode = teaException.getCode();
            String errMsg = teaException.getMessage();

            if (StrUtil.isAllNotBlank(errCode, errMsg)) {
                log.error("钉钉授权认证失败： code：{}，message：{}", errCode, errMsg);
                throw new ServiceException(errMsg);
            } else {
                log.error("钉钉授权认证失败", teaException);
                throw new ServiceException("发生未知异常，钉钉授权认证失败");
            }

        }
    }

    /**
     * 通过免登码获取用户信息
     * https://open.dingtalk.com/document/isvapp/obtain-the-userid-of-a-user-by-using-the-log-free
     */
    public static OapiV2UserGetuserinfoResponse.UserGetByCodeResponse getUserInfo(String corpAccessToken, String code) {
        try {
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/user/getuserinfo");
            OapiV2UserGetuserinfoRequest req = new OapiV2UserGetuserinfoRequest();
            req.setCode(code);
            OapiV2UserGetuserinfoResponse rsp = client.execute(req, corpAccessToken);
            return rsp.getResult();
        } catch (ApiException e) {
            log.error("通过免登码获取用户信息失败");
            throw new ServiceException("通过免登码获取用户信息失败");
        }
    }


    private static com.aliyun.dingtalkcontact_1_0.Client getClient1() throws Exception {
        Config config = new Config();
        config.protocol = "https";
        config.regionId = "central";
        return new com.aliyun.dingtalkcontact_1_0.Client(config);
    }

    private static com.aliyun.dingtalkoauth2_1_0.Client getClient2() throws Exception {
        Config config = new Config();
        config.protocol = "https";
        config.regionId = "central";
        return new com.aliyun.dingtalkoauth2_1_0.Client(config);
    }

}
